
/* 
 *          Gnome Rhythm, by stoney -- lstone@gate.net
 *
 *  This is grhythm.c  - an attempt to port a cardiac rhythm recognition
 *  application that I wrote in TurboC to Gnome (groan) and Linux.
 *
 *
 */
 
 #include <gnome.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <libgnomeui/gnome-init.h>
 #include "grhythm.h"
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
#ifndef __hpux
 #include <wait.h>
#endif
 #include <errno.h>
 #include <sys/time.h>
 #include <sys/types.h>

 FILE *datafile;
 GtkWidget *appwindow;
 GtkWidget *drawing_area;
 GtkWidget *box2;

 static GdkPixmap *pixmap = NULL;   /* Backing pixmap for drawing area */
 GdkRectangle rect;
 struct GdkPoint mypoints[MAXPOINTS];
 struct GdkPoint drawpoints[MAXPOINTS];
 struct timeval time_s;
 
 static GtkItemFactoryEntry menu_items[] = {
   { "/_File",              NULL,        NULL,           0,  "<Branch>" },
   { "/File/Exit",         "<control>X", gtk_main_quit,  0,   NULL },
   { "/_View",              NULL,        NULL,           0,  "<Branch>"},
   { "/View/Tutorial",      NULL,        grhythm_do_tutorial,  0,   NULL },
   { "/View/EKG Basics",    NULL,        grhythm_do_basics,    0,   NULL },
   { "/View/Flashcard",     NULL,        grhythm_do_practice,  0,   NULL },      
   { "/_Help",              NULL,        NULL,           0,  "<LastBranch>" },
   { "/_Help/User's Guide", NULL,        not_imp_cb,     0,   NULL },
   { "/_Help/About",        NULL,        about_cb,       0,   NULL }, 
 };


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *              callbacks and utility functions                              *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static void not_imp_cb(GtkWidget *widget, gpointer data)
{    
   gnome_ok_dialog("Sorry, this feature is not yet implemented.");
} 

static void error_dialog_cb(char *message)
{    
   gnome_error_dialog(message);
}

void delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit ();
}

void gtk_quit()
{
   gtk_exit (0);
}

static gint configure_event(GtkWidget *widget, GdkEventConfigure *event )
{
   if (pixmap)
       gdk_pixmap_unref(pixmap);
    
   pixmap = gdk_pixmap_new(widget->window, widget->allocation.width,
			      100, -1);
   gdk_draw_rectangle (pixmap, widget->style->black_gc, TRUE, 0, 0,
		       widget->allocation.width, widget->allocation.height);
   rect.x = event->x;
   rect.y = event->y;
   rect.width = event->width;
   rect.height = event->height;
   return TRUE;
}


static gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
  grhythm_draw_strip(GTK_WIDGET(widget) );
  return FALSE;
}


void grhythm_delay(gulong msec) 
{
   struct timeval timeout;

   timeout.tv_sec = msec/1000000L;
   timeout.tv_usec = msec - (timeout.tv_sec * 1000000L);
   select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &timeout);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                         grhythm_draw_strip                                *
 *                                                                           *
 *            called by inc_strip, dec_strip, and expose_event               *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static gint grhythm_draw_strip(GtkWidget *widget)
{
  GdkColor color;
  GdkColormap* color_map;
  color_map =gdk_colormap_get_system();

  grdata.gc = gdk_gc_new(widget->window);

  color.red=0;
  color.green=65535;
  color.blue=0;

  if (!gdk_colormap_alloc_color(color_map, &color, FALSE, TRUE))
     g_print("grhythm: Could not get color\n");
  
  gdk_gc_set_foreground(grdata.gc, &color); 

  gdk_draw_rectangle(widget->window, widget->style->white_gc, TRUE,
                      0, 102, rect.width,
                      rect.height -100 );   

  gdk_draw_rectangle(widget->window, widget->style->black_gc, TRUE,
		      0, 0, rect.width, 100 );

  grdata.elements = grhythm_get_segment(GTK_WIDGET(widget));   /* calls get_segment */

  if ((grdata.elements == 0)) {       /* signals end of section */
     grhythm_static_toggle(widget);
     grdata.eos = TRUE;
     return 0;
     }
  else 
     return grdata.elements;
     
}   /* end of draw_strip */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                           grhythm_get_segment                             *
 *                       called by grhythm_draw_strip                        *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static gint grhythm_get_segment(GtkWidget *widget) 
{
  char filename[13];
  gint drawing_size = 0;  /*  horizontal size of segment                  */  
  gint x_pos;             /*  placekeeper in array, the return value      */
  gint x_count;           /*  the absolute horizontal position in pixels  */
  gint ypix = 50;         /*  vertical component of x-y pairs             */
  gint ch;
  gint ret;
  gint i;

  switch(grdata.section){
    case BASICS:   sprintf(filename, "basics.dat"); break;
    case TUTORIAL: sprintf(filename, "rhythm.dat"); break;
    case PRACTICE: sprintf(filename, "practice.dat"); break;
    default:       sprintf(filename, "basics.dat"); break;
    }

  if ((datafile = fopen(filename, "r")) == NULL) {
      g_print("grhythm: Unable to open datafile\n");
      error_dialog_cb("A Grhythm error has occurred: Can not find
                       data file");
      gtk_main_quit();      
      }

  grdata.total_segments = 0;
  
  while( ((ch = fgetc(datafile)) != EOF) )
     if ((ch == '$'))
       grdata.total_segments++;

  /*
   *  basics and tutorial strips start at 000 
   *  practice strips start at 001
   */
  
  if ((grdata.section != PRACTICE)) {
     if ((grdata.segment == grdata.total_segments - 1))
         grdata.eos = TRUE;
      }
  else if ((grdata.section == PRACTICE)) {
     if ((grdata.segment == grdata.total_segments)) // -1
         grdata.eos = TRUE;
     } 

  rewind(datafile);
 
  i = 0;
  while (i < grdata.segment) {
    ch = fgetc(datafile);
    if ((ch == '@'))
      i++;
    }
      
  while( ((ch = fgetc(datafile)) != '$' ))
    switch(ch) {
       case '3': grdata.elements++; drawing_size++; break;
       case '6': grdata.elements++; drawing_size++; break;
       case '9': grdata.elements++; drawing_size++; break;
       case '2': grdata.elements++; break;
       case '8': grdata.elements++; break;
       default:  break;
       }

  for( x_pos = 0; x_pos <= MAXPOINTS; x_pos++ ) {
       mypoints[x_pos].y = 0;
       mypoints[x_pos].x = 0;
       }
      
  rewind(datafile);
  
  i = 0;
  while (i < grdata.segment) {
    ch = fgetc(datafile);
    if ((ch == '@'))
      i++;
    }  
  
  x_count = 1;
  x_pos = 0;

  while(( ch = fgetc(datafile) ))
     {
      if (ch == EOF) {
        g_print("EOF\n");
        return 0;
        } 
      
      if ((ch == '$')) 
           {
            grdata.ds = drawing_size;            
            grhythm_draw_legend(GTK_WIDGET(widget));   /* calls draw_legend */
            fclose(datafile);
            ret = grhythm_fill_strip(drawing_size, grdata.elements);  /* calls fill_strip */
            return(ret);  /* returns to draw strip */
            }

      else {
        switch (ch) {
            case '2':  ypix++;  x_count--; break;  
            case '3':  ypix++;  break;
            case '6':  break;
            case '8':  ypix--;  x_count--; break;           
            case '9':  ypix--;  break;
            case '|':  x_count--; break;
            default:   x_count--; break;
            }
        mypoints[x_pos].x = x_count;
        mypoints[x_pos].y = ypix;

        x_pos++;
        x_count++;     
        }      
     }

  return 1;  
}            /* returns to draw strip */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                         grhythm_draw_legend                               *
 *                     called by grhythm_get_segment                         *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static void grhythm_draw_legend(GtkWidget *widget)
{
   /* One way to do this is to leave the file open and let
     a text drawing function handle it. There should
     probably be one function for each of the sections.
     The basics section will need a second small window
     to draw the colored parts in, and the practice
     section will not display anything but a page
     number, 3 distractors, and the real strip name
     from the datafile.  */
   

  FILE *namefile;
  gchar name[71];
  GdkColor color;
  GdkColormap* color_map;
  gint usecolor;
  gint docolor = FALSE;
  gint width, ch, i, x, y;
  gint x_start, y_start;
  gint lineheight, indent;
  gint message = FALSE;
  gint count = 0;
  gint space[4];           /* array to hold four positions */
  gint spacecounter = 0;
  gint random_seed;
  gint rand1=0, rand2=0, rand3=0;
  gint flash;
  gchar line_out[400];
  gchar nameholder[71];
  struct timeval tv;
  gchar buf[4];
  gchar *p;
  gchar temp[2], title[60], line[600];
  
  gchar decoy1[75];   /* strings for decoy rhythm names */
  gchar decoy2[75];
  gchar decoy3[75];
  gchar string[65];
  
  strcpy(buf,"");
  strcpy(temp,"");
  strcpy(title, "");
  strcpy(line, "");
  strcpy(decoy1,"");
  strcpy(decoy2,"");
  strcpy(decoy3,"");
  strcpy(string, "");
   

  
  color_map =gdk_colormap_get_system();
  
  grdata.font = gdk_font_load(
          "-*-helvetica-medium-r-normal-*-*-140-*-*-p-*-iso8859-1");

  /* grab the title and strip number first */

  if ((grdata.section != PRACTICE )) {     /* not in practice strips */
      ch = fgetc(datafile);
      while((ch != '/')){           /* strip number */
	  sprintf(temp, "%c", ch);
	  strcat(buf, temp);        /* I miss TurboC */
	  ch = fgetc(datafile);
	   }
      ch = fgetc(datafile);
      if ((ch == 't')) {           /* strip title */
          ch = fgetc(datafile);
          while ((ch != '/' && ch != EOF)) {
               sprintf(temp, "%c", ch);
               strcat(title, temp);   /* I miss TurboC */
               ch = fgetc(datafile);
               }
          if ((ch == EOF))
            grdata.eos = TRUE;
          } 
      }

  else if ((grdata.section == PRACTICE)) /* in practice strips */
     {
   
      
      gdk_draw_rectangle(widget->window, widget->style->white_gc, TRUE,
                          0, 102, rect.width, rect.height - 10 );   
     
 
        
      sprintf(string, "Score is %.0f", grdata.score * 100);
           
      gdk_draw_string(widget->window, grdata.font, widget->style->black_gc, 
                      rect.width - 90, 393, string); 
      
 
      ch = fgetc(datafile);    
      if ((ch == '^'))  
          ch = fgetc(datafile);
          
      while ((ch != '>')) {               /* strip number */
          sprintf(temp, "%c", ch);
          strcat(buf, temp);           
          ch = fgetc(datafile);
          }

      if ((ch == '>')){                      /* strip title */
          ch = fgetc(datafile);
          while ((ch != '@' && ch != EOF)) {
             sprintf(temp, "%c", ch);
             strcat(title, temp);         /* title contains practice strip name */
             ch = fgetc(datafile);
             }
          if ((ch == EOF))
            grdata.eos = TRUE;
          }   

 
  
   /* start getting random numbers */

   for(i = 0; i < 5; i++)
	space[i] = 0;
 
   gettimeofday(&tv, NULL);
   
   srand(tv.tv_usec);

   while(space[3] == 0)
	{
	random_seed = (rand() % 100);
	
	if((random_seed >= 0) && (random_seed < 25))
	  flash = 1;
	if((random_seed > 24) && (random_seed < 50))
	  flash = 2;
	if((random_seed > 49) && (random_seed < 75))
	  flash = 3;
	if(random_seed > 74)
	  flash = 4;

	if(space[0] == 0)
	  {
	  space[0] = flash;
	  continue;
	  }
	if((space[0] > 0) && (space[1] == 0))
	  if (flash != space[0])
	    {
	    space[1] = flash;
	    continue;
	    }
	if((space[1] > 0) && (space[2] == 0))
	  if ((flash != space[0]) && (flash != space[1]))
	    {
	    space[2] = flash;
	    continue;
	    }
	if((space[2] > 0) && (space[3] == 0))
	  if ((flash != space[0]) && (flash != space[1]) && (flash != space[2]))
	    {
	    space[3] = flash;
	    continue;
	    }
	}

    /* Now we have 4 strip positions */
    
   
    

   /* open names file and get number of names */

   namefile = fopen("strips.dat","r");

   /* count available number of strip names */

   flash = 0;
   
   if(namefile != NULL)
    while(fgets(name,70,namefile) != NULL)
	 flash++;

     /* start of decoy name loop - get three decoy names */

   for(spacecounter = 0; spacecounter < 3; spacecounter++) 	{
	
	random_seed = (rand() % (flash -1));

	if(random_seed == 0)  {
	  spacecounter--;        /* keep trying */
	  continue;
	  }
	
	if(random_seed == rand1 || random_seed == rand2 || random_seed == rand3){
	   spacecounter--;     /* keep trying if number already used */
	   continue;
	   }
	
	switch(spacecounter)  {
	  case 0: rand1 = random_seed; break;
	  case 1: rand2 = random_seed; break;
	  case 2: rand3 = random_seed; break;
	  }
	
	rewind(namefile);

	/* get the rhythm name at the line number random_seed */

	for (count = 1; count <= random_seed; count++) {
	  strcpy(nameholder,"");
	  fgets(nameholder, 70, namefile);
	  }

        strcpy(name,"");

	/* see if it's blank */

	if(nameholder[0] == ' ') {
	  spacecounter = -1;
	  count = random_seed; /* random_seed was last line in file */
	  }
	
	/* remove trailing return */
	
        strncat(name, nameholder, strlen(nameholder) - 1);
	
	/* make sure it's not the same as the rhythm name 
            (spacecounter = -1 resets the for loop)      */

	if(strcmp(name,line_out) == 0)
            spacecounter = -1;

	/* and not using a blank line */

	if(name[0] == ' ')
            spacecounter = -1;
	   
	/* and see if it's already in use as a decoy */

	if(strncmp(name,decoy1,strlen(name)-1) == 0)
	  spacecounter = -1;
	  
	if(strncmp(name,decoy2,strlen(name)-1) == 0) 
	  spacecounter = -1;
	  
	if(strncmp(name,decoy3,strlen(name)-1) == 0)
	  spacecounter = -1;
	 
	if(spacecounter == 2)
	  {
	  if(decoy1[0] == ' ')
	    spacecounter = -1;

	  if(decoy2[0] == ' ')
	    spacecounter = -1;
	  }

	/* before putting it in a decoy */

	switch(spacecounter)  {
	  case 0: strcpy(decoy1,name);break;
	  case 1: strcpy(decoy2,name);break;
	  case 2: strcpy(decoy3,name);break;
	  }
	}
    fclose(namefile);
    
    /* and display the labels */
    
    gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                    50, 195, "A.");
    gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                    50, 225, "B.");    
    gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                    50, 255, "C.");    
    gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                    50, 285, "D.");

    /* and the choices */
    
    i = 0;
    y = 195;
    while ((i < 4)) {
       flash = space[i];
       switch(flash) {
          case 1: 
            gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                            75, y, decoy1);
            break;
          case 2: 
            gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                            75, y, decoy2);
            break;
          case 3:
            gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                            75, y, decoy3);
            break;
          case 4: 
            gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                            75, y, title);
            break;
          default:
            g_print("gRhythm: Error in flashcard section");
            break;
            }
          
       y += 30;
          
       i++;
       }
       
     /* find out where the title was */
     
     i = 0;
     
     while (i < 4) { 
        if (space[i] != 4)
           i++;
        else {
           grdata.current = i + 1;
           i = 5;
           }
         }
        
     /* and show the flashcard buttons */
  
     if ((grdata.section == PRACTICE))
           gtk_widget_show(box2);
     
    /* remember, you're pushing this bracket */
    }

  
  if ((grdata.section != PRACTICE))
      gtk_widget_hide(box2);
      
      
  /* for tutorial and basics, put the title */

  width = gdk_string_width(grdata.font, title);

  if ((grdata.section != PRACTICE))
     gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                     (rect.width / 2 - width / 2), 133, title);
                  
  /* in all cases, put the strip number */
                
  gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                   rect.width - 60, 133, buf); 
  
  /* then grab the text for non-flashcard page */

  ch = fgetc(datafile);  
  x_start = 20;
  y_start = 163;
  lineheight = 23;
 
  if ((grdata.section != PRACTICE))
     {
     while ((message != TRUE))   {
         if ((ch == ':')) {             /* indent - don't change y */
            if ((line[0] == '/')) {
               g_strreverse(line);
               p = strtok(line, "/");
               g_strreverse(line);
               ch = fgetc(datafile);
               }  
            if ((gdk_string_width(grdata.font, line) < rect.width -40))
              gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                              x_start, y_start, line);
            else 
              y_start = grhythm_wrap_text(GTK_WIDGET(widget), line, x_start,
                                           y_start, rect.width - 40, FALSE, lineheight); 
            strcpy(line, "");
            indent = TRUE;               
            }
         else if ((ch == '@')) {     /* end of segment */
            if ((indent == TRUE))
                x_start += 100;
                
            if ((line[0] == ':')) {
               g_strreverse(line);
               p = strtok(line, ":");
               g_strreverse(line);
               ch = fgetc(datafile);
               }  
            
            if ((line[0] == '/')) {
               g_strreverse(line);
               p = strtok(line, "/");
               g_strreverse(line);
               ch = fgetc(datafile);
               }
            
            if ((gdk_string_width(grdata.font, line) + x_start < rect.width -40)) {
               gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                               x_start, y_start, line);
               y_start += lineheight;
               }                             
            else                    
               y_start = grhythm_wrap_text(GTK_WIDGET(widget), line, x_start,
                                            y_start, rect.width - 40, indent,
                                            lineheight);
            indent = FALSE;
            strcpy(line, "");
            x_start = 20;
            message = TRUE;    /* exits */
            }    
         else if ((ch == '/')){
            if ((indent == TRUE))
               x_start += 100;
            
            if ((line[0] == '/')) {  /* new line */
               g_strreverse(line);
               p = strtok(line, "/");
               g_strreverse(line);
               ch = fgetc(datafile);
                }
         
            if ((line[0] == ':')) {   /* offset */
               g_strreverse(line);
               p = strtok(line, ":");
               g_strreverse(line);
               ch = fgetc(datafile);
                }           
           
            if ((gdk_string_width(grdata.font, line) + x_start < rect.width - 40)) {
                gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                                x_start, y_start, line);
                y_start += lineheight;
                indent = FALSE;
                strcpy(line, "");
                x_start = 20;
                }
            else {
                y_start = grhythm_wrap_text(GTK_WIDGET(widget), line, x_start,
                                         y_start, rect.width - 40, FALSE,
                                         lineheight);
                indent = FALSE;
                strcpy(line, "");
                x_start = 20;
                }
            }    
        sprintf(temp, "%c", ch);
        strcat(line, temp);
        ch = fgetc(datafile);
        
        } 
    }

  if ((ch == EOF))
     grdata.eos = TRUE;


  /*  For basics section, a subwindow is needed to display parts of
      of the waveform.  */

  x = 30;
  y = 400; 
  
  if ((grdata.section == BASICS))
    {
     i= 0;
     usecolor = FALSE;
     rewind(datafile);
     
     while (i <= grdata.segment) {  
        ch = fgetc(datafile);
        if ((ch == '@'))      /* move forward in file looking for strips */
          i++;
        if ((i == grdata.segment -1 ))
          if ((ch == '~'))     /* see if subwindow is needed for particular strip */
             usecolor = TRUE;
        }

     
     usecolor = TRUE;
     
     if ((usecolor == TRUE)) 
          gdk_draw_rectangle(widget->window, widget->style->black_gc,TRUE,
		              20, 350, grdata.ds + 20, 90 );
     rewind(datafile);
     i = 0;
     while ((i < grdata.segment)) {      /* get to right place in file */
        ch = fgetc(datafile);
        if ((ch =='@'))
           i++;
        }

     while ((ch != '$' && usecolor == TRUE ))
       {
        ch = fgetc(datafile);
        switch (ch) {
            case '2':  y++; break; 
            case '3':  y++; x++; break;
            case '6':  x++; break;
            case '8':  y--; break;           
            case '9':  y--; x++;  break;
            case '~':  
                if(( docolor == FALSE )){
                   color.red=65535;
                   color.green=0;
                   color.blue=0;
                   if (!gdk_colormap_alloc_color(color_map, &color, FALSE, TRUE))
                      g_print("grhythm: Could not get color\n");
                   gdk_gc_set_foreground(grdata.gc, &color);     
                   docolor = TRUE;
                   }
                else if (( docolor == TRUE )) {
                   docolor = FALSE;
                   color.red=0;
                   color.green=65535;
                   color.blue=0;
                   if (!gdk_colormap_alloc_color(color_map, &color, FALSE, TRUE))
                      g_print("grhythm: Could not get color\n");
                   gdk_gc_set_foreground(grdata.gc, &color);     
                   docolor =  FALSE;
                   }                  
                break;
            default:  break;
            }

        gdk_draw_point(widget->window, grdata.gc, x, y); 
       }
     }   
  
  usecolor = FALSE;
  
  color.red=0;
  color.green=65535;
  color.blue=0;

  if (!gdk_colormap_alloc_color(color_map, &color, FALSE, TRUE))
     g_print("grhythm: Could not get color\n");
  
  gdk_gc_set_foreground(grdata.gc, &color);   
  
}  /* draw_legend returns void */







static void button_press_event (GtkWidget *widget, gpointer data)
{
   
   //gint user_try = (gint) data;
   
   grdata.num_attempted++;
   
   if (( grdata.current == (gint) data ))
        grdata.num_correct++;
     
  
   grdata.score = grdata.num_correct / grdata.num_attempted;
   
   if ((grdata.eos == TRUE))
       return;
     
   if ((grdata.strip_running == OFF))
       grhythm_static_toggle(GTK_WIDGET(widget));

   grdata.section = PRACTICE;
   grdata.current = 0;
   
   grdata.segment++;
   
   grhythm_draw_strip(GTK_WIDGET(drawing_area));   
   
   return;
}














/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                         grhythm_fill_strip                                *
 *                    called by grhythm_get_segment                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static gint grhythm_fill_strip(gint drawing_size, gint elements)
{
  gint pos;
  gint xcount;
  gint i;
  
  if ((mypoints[0].y == 0))  {
     g_print("grhythm_fill_strip: Array does not contain any y values\n");
     error_dialog_cb("A Grhythm error has occurred: in function grhythm_fill_strip,
                     struct does not contain any y values");
     return 0;
     }

   /* see where the first segment ended */ 
   pos = 0;
   while ((mypoints[pos].x != 0)){
    /*  g_print("mypoints[%d] contains %d x  %d y \n", pos, 
                 mypoints[pos].x, mypoints[pos].y); */
      pos++;
      }
   pos--;

   /* fill in rest of strip */
   i = 1;
   while (i < MAXPOINTS ) { 
      mypoints[pos + i].x = (mypoints[i -1].x) + drawing_size;
      mypoints[pos + i].y =  mypoints[i-1].y;
      
      /* g_print("mypoints[%d] contains %d x  %d y \n", pos + i,
               mypoints[pos+ i].x, mypoints[pos+ i].y); */
      
      if ((mypoints[pos+i].x <= MAXPOINTS))
         xcount = pos + i; 
      if ((pos + i == MAXPOINTS - 1))
         i = MAXPOINTS;
      i++;
      }    
   return(xcount); /* returns to get_segment */
}



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                         grhythm_animate_strip                             *
 *                    idle function that moves the strip                     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
             /* problems are usually here */

static gboolean grhythm_animate_strip(GtkWidget *widget)
{
  gint ret, lineno, i;
  static gint count=1;
  gint diff;
  static gint current = 1;
  static GTimer *timer;

  timer = g_timer_new();
  
  if ((current != grdata.section)) {    /* section changed by the user */
    current = grdata.section;
    grhythm_draw_strip(GTK_WIDGET(widget));
    }
  
  if ((grdata.strip_running == OFF))    /* don't animate if frozen */
    return FALSE;

  if ((count >= grdata.ds)) {           /* end of segment */
    i = 0;
    while ((i < MAXPOINTS)) {          /* reset to original */
      drawpoints[i].x = mypoints[i].x;
      drawpoints[i].y = mypoints[i].y;
      i++;
      }
    count = 0;
    }  
  
  else if ( (count >= 1) && (count <  grdata.ds) ) {
    i=0;
    while((mypoints[i].x  <  count + 1))
      i++;
    diff = i;  /* diff is the offset */
    i = 0;
    while ((i < MAXPOINTS - 200))  {
      drawpoints[i].x = mypoints[diff].x - count;
      drawpoints[i].y = mypoints[diff].y;
      i++;
      diff++;
      } 

    /* something wrong with the datafile */
   
    if ((drawpoints[0].x != 1 )) {   /* stop the animation  */
       g_print("drawpoints[0] not 1: it contains %d x  %d y \n",
                drawpoints[0].x, drawpoints[0].y);  
       return FALSE;
      }
    }
  

  
  while ((g_timer_elapsed(timer, NULL) < .012))
    grhythm_delay ( 50 );
  
  g_timer_stop(timer);     

          
  gdk_draw_rectangle(pixmap, widget->style->black_gc ,
                          TRUE, 0, 0, rect.width, 100 );
      
  if ((grdata.show_grid == ON)) {
     for(ret = 1, lineno = 1; ret < rect.width; ret++)
        if (ret % 14 == 0)  {
            gdk_draw_line(pixmap, widget->style->fg_gc[GTK_STATE_INSENSITIVE],
                          ret, 1, ret, 98);
            lineno++;
            if (lineno % 5 == 0) { 
               gdk_draw_line(pixmap, widget->style->white_gc,
                          ret-14, 0, ret-14, 4);              
	       gdk_draw_line(pixmap, widget->style->white_gc,
                          ret-14, 100, ret-14, 96);
               }
            }
     for(ret = 1; ret < 90; ret++)    
         if (ret % 14 == 0)
             gdk_draw_line(pixmap, widget->style->fg_gc[GTK_STATE_INSENSITIVE],
                           1, ret, rect.width, ret);
     }
  
  gdk_draw_points(pixmap, grdata.gc, (GdkPoint *)drawpoints, grdata.elements ); 
  

 
  gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                  pixmap, 0, 0, 0, 0, rect.width, 100);
                      
     
  count++;
  g_timer_start(timer);
  
  /* return FALSE to stop, TRUE to continue */

  return TRUE;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                        toolbar functions                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


static void grhythm_grid_toggle(GtkWidget *widget)
{
   if ((grdata.show_grid == ON ))
      grdata.show_grid = OFF;
   else 
      grdata.show_grid = ON;
}

static void grhythm_static_toggle(GtkWidget *widget)
{
   if ((grdata.strip_running == ON ))
      grdata.strip_running = OFF;
   else {  
      gtk_idle_add((GtkFunction)grhythm_animate_strip, drawing_area);
      grdata.strip_running = ON;
      }
   return;
}

static void grhythm_inc_strip(GtkWidget *widget, GtkWidget *drawspace)
{
  if ((grdata.eos == FALSE))
       grdata.segment++;
  else 
       return;
  
  if ((grdata.section == PRACTICE))
      return;
  
  if ((grdata.strip_running == OFF))
       grhythm_static_toggle(GTK_WIDGET(widget));

  grhythm_draw_strip(GTK_WIDGET(drawspace)); 
}

      
static void grhythm_jump_forward(GtkWidget *widget, GtkWidget *drawspace)
{
  if ((grdata.section == PRACTICE)) 
     return;

  if ((grdata.eos == FALSE)) {
      grdata.segment += 5;
      if ((grdata.total_segments <= grdata.segment))
         grdata.segment = grdata.total_segments -1;
      grhythm_draw_strip(GTK_WIDGET(drawspace));
      }
  else 
       return;
}

     
static void grhythm_jump_reverse(GtkWidget *widget, GtkWidget *drawspace)
{
  if ((grdata.section == PRACTICE))
     return;
  
  if ((grdata.strip_running == FALSE))
      grhythm_static_toggle(widget);      
  
  if ((grdata.segment == 1))
     return;

  if ((grdata.segment > 5))  
      grdata.segment -= 5;
  else
      grdata.segment = 1;

  grdata.eos = FALSE;
  gtk_idle_add((GtkFunction)grhythm_animate_strip, drawing_area);
  grhythm_draw_strip(GTK_WIDGET(drawspace));
  return;
}

static void grhythm_dec_strip(GtkWidget *widget, GtkWidget *drawspace)
{
  if ((grdata.section == PRACTICE))
      return;
  
  if ((grdata.strip_running == OFF))
      grhythm_static_toggle(widget);

  if ((grdata.segment == 1))
     return;
   
  if ((grdata.segment > 1)){
      grdata.segment--;
      gtk_idle_add((GtkFunction)grhythm_animate_strip, drawing_area);
      grdata.eos = FALSE;
      grhythm_draw_strip(GTK_WIDGET(drawspace));
      }
  else
      return;
}



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                           menu functions                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void get_main_menu( GtkWidget  *window, GtkWidget **menubar )
{
   GtkItemFactory *item_factory;
   GtkAccelGroup *accel_group;
   gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

   accel_group = gtk_accel_group_new ();
   item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
                                        accel_group);
   gtk_item_factory_create_items (item_factory, nmenu_items, 
                                  menu_items, NULL);
   gtk_accel_group_attach (accel_group, GTK_OBJECT(window));

   if (menubar)
     *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

static void grhythm_do_basics(GtkWidget *widget)
{
  grdata.segment = 1;  /* first strip in section */
  grdata.section = BASICS;
  gtk_window_set_title (GTK_WINDOW(appwindow), "gRhythm: basics");
  
  if ((grdata.eos == TRUE)) {
     gtk_idle_add((GtkFunction)grhythm_animate_strip, drawing_area);
     grdata.eos = FALSE;
     grdata.strip_running = TRUE;
     }
}

static void grhythm_do_tutorial(GtkWidget *widget)
{
  grdata.segment = 1;
  grdata.section = TUTORIAL;
  gtk_window_set_title (GTK_WINDOW(appwindow), "gRhythm: tutorial");
  if ((grdata.eos == TRUE)) {
     gtk_idle_add((GtkFunction)grhythm_animate_strip, drawing_area);
     grdata.eos = FALSE;
     grdata.strip_running = TRUE;
     }
}

static void grhythm_do_practice(GtkWidget *widget)
{
  grdata.segment = 1;
  grdata.section = PRACTICE;
  gtk_window_set_title (GTK_WINDOW(appwindow), "gRhythm: flashcard"); 
  
  grdata.num_attempted = 0;
  grdata.current = 0;
  grdata.num_attempted =0;
  grdata.num_correct = 0;
  grdata.score = 0;   
     
  if ((grdata.eos == TRUE)) {
     gtk_idle_add((GtkFunction)grhythm_animate_strip, drawing_area);
     grdata.eos = FALSE;
     grdata.strip_running = TRUE;
     }
}

static void about_cb (GtkWidget *widget, gpointer data)
{
   GtkWidget *about;
   static const gchar *author[]={ "\"Stoney\"  <lstone@gate.net>",
                                  NULL}; 
   about=gnome_about_new(_("Gnome Rhythm"), VERSION,
                          _("(C) 2000 the Free Software Foundation"),
                  author, _("A Gnome port of a cardiac"
                            " rhythm recognition program"
                            " written for DOS"), NULL);
   gtk_widget_show (about);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                         grhythm_wrap_text                                 *
 *                   called by grhythm_draw_legend                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
static gint
grhythm_wrap_text(GtkWidget *widget, gchar *string, gint x_start, gint y_start,
                       gint linewidth, gint indent, gint lineheight)
{
  gchar lineholder[250];
  gchar *p;
  gint linecounter = 0;

  strcpy(lineholder, "");

       /* doesn't need wrapping */ 
  
  if(( gdk_string_width(grdata.font, string)) + x_start < rect.width - 40)  {  
     gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                    x_start, y_start, string);
     return lineheight + y_start;             /* this shouldn't happen */
     }
  else {                          /* does need wrapping */
    
    if (!string[0])  
         return y_start;            /* see if empty string */

    p = strtok(string, " ");      /* look for spaces */

    if (p != NULL) {              /* found some spaces */
        strcat(lineholder, p);      /* append up to the space */
        strcat(lineholder, " ");    /* and put the space in   */
        }
   
    while ((gdk_string_width(grdata.font, lineholder)) <= linewidth && p != NULL) {
        p = strtok(NULL, " "); 
        strcat(lineholder, p);     /* find and add a word at a time */
        strcat(lineholder, " ");
        p = strtok(NULL, " ");

    while (p != NULL) {  
	if((gdk_string_width(grdata.font, lineholder) + gdk_string_width(grdata.font, p) + 
	     x_start)<= linewidth) {
	    strcat(lineholder, p);
	    strcat(lineholder, " ");
	    p = strtok(NULL, " ");
	    }
	else  {
            gdk_draw_string(widget->window, grdata.font, widget->style->black_gc,
                            x_start, y_start, lineholder);
	    linecounter++;
            strcpy(lineholder, "");
	    strcat(lineholder, p);
	    strcat(lineholder, " ");
	    y_start += lineheight;
	    p = strtok(NULL, " ");
	    }
	 }
	 
        if (p == NULL) {                   /* stopped before end of line, so... */
	   gdk_draw_string(widget->window, grdata.font, widget->style->black_gc, 
	                    x_start, y_start, lineholder);
	y_start += lineheight;        
	return (y_start);
        }
      }
   }
  return (y_start);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                                main                                       *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int main(int argc, char **argv)
{

  GtkWidget *menubar;
  GtkWidget *toolbar;
  GtkWidget *icon;
  gint idle_deeds;
  GtkWidget *box1;
  GtkWidget *button1, *button2, *button3, *button4; 
  
  
  grdata.segment = 2;
  grdata.strip_running = TRUE;
  grdata.show_grid = TRUE;

  gnome_init ("grhythm", VERSION, argc, argv);
  
  appwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_title (GTK_WINDOW (appwindow), "gRhythm: tutorial");

  gtk_signal_connect (GTK_OBJECT (appwindow), "delete_event",
                      GTK_SIGNAL_FUNC (delete_event), NULL);

  gtk_signal_connect (GTK_OBJECT (appwindow), "destroy",
		      GTK_SIGNAL_FUNC (gtk_quit), NULL);
    
  box1 = gtk_vbox_new(FALSE, 10);
  gtk_widget_set_usize(box1, 590, 570);
  gtk_container_set_border_width (GTK_CONTAINER (box1), 3);
  gtk_container_add (GTK_CONTAINER (appwindow), box1);  

  /* menu */

  get_main_menu (appwindow, &menubar);
  gtk_box_pack_start (GTK_BOX (box1), menubar, FALSE, TRUE, 0);
  gtk_widget_show (menubar);

  /* toolbar */

  toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,  GTK_TOOLBAR_ICONS);
  
  gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
  
  gtk_container_set_border_width (GTK_CONTAINER(toolbar) , 1 );
  gtk_toolbar_set_space_size (GTK_TOOLBAR( toolbar ), 1 );

  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  icon = gnome_pixmap_new_from_file("pause.xpm");
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),NULL, "Toggle static strip", "Private",
                          GTK_WIDGET(icon),GTK_SIGNAL_FUNC(grhythm_static_toggle), toolbar);

  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
  
  icon = gnome_pixmap_new_from_file("gr_grid.xpm");
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),NULL,"Toggle background grid","Private",
                          GTK_WIDGET(icon),GTK_SIGNAL_FUNC(grhythm_grid_toggle), toolbar);
                          
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));                          

  /* drawing area */
  
  drawing_area = gtk_drawing_area_new();     

  /* more toolbar buttons */
  
  icon = gnome_pixmap_new_from_file("retroceder.xpm");                          
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Go back 5 strips","Private",
                          GTK_WIDGET(icon), GTK_SIGNAL_FUNC(grhythm_jump_reverse), drawing_area);

  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
  
  icon = gnome_pixmap_new_from_file("retrocar.xpm");                          
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Go to previous strip","Private",
                          GTK_WIDGET(icon), GTK_SIGNAL_FUNC(grhythm_dec_strip), drawing_area);

  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  icon = gnome_pixmap_new_from_file("tocar.xpm");
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Advance to next strip","Private",
                          GTK_WIDGET(icon),GTK_SIGNAL_FUNC(grhythm_inc_strip), drawing_area );

  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
            
  icon = gnome_pixmap_new_from_file("avanzar.xpm");
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Advance five strips","Private",
                       GTK_WIDGET(icon),GTK_SIGNAL_FUNC(grhythm_jump_forward), drawing_area );

  gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar),GTK_RELIEF_NONE);  
  gtk_box_pack_start (GTK_BOX (box1), toolbar, FALSE, TRUE, 0);
  gtk_widget_show(toolbar); 
  
  /* and drawing area again */

  gtk_widget_set_usize(drawing_area, 580, 450);
  gtk_box_pack_start(GTK_BOX (box1), drawing_area, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
		      (GtkSignalFunc) expose_event, NULL); 
  gtk_signal_connect (GTK_OBJECT(drawing_area),"configure_event",
		      (GtkSignalFunc) configure_event, NULL); 

  /* and selector buttons for flashcard section */
  
  box2 = gtk_hbox_new(FALSE, 10);
  
  gtk_widget_set_usize(box2, 580, 30);
  gtk_container_set_border_width (GTK_CONTAINER (box2), 3);
  gtk_container_add (GTK_CONTAINER (box1), box2); 
  
  button1 = gtk_button_new_with_label("A");
  gtk_box_pack_start (GTK_BOX (box2), button1, TRUE, TRUE, 0);
  gtk_widget_realize(button1);
  gtk_widget_show(button1); 
  gtk_signal_connect( GTK_OBJECT(button1), "clicked",
                      GTK_SIGNAL_FUNC (button_press_event),
                      (gpointer) 1); 

  button2 = gtk_button_new_with_label("B");
  gtk_box_pack_start (GTK_BOX (box2), button2, TRUE, TRUE, 0);
  gtk_widget_realize(button2);
  gtk_widget_show(button2); 
  gtk_signal_connect( GTK_OBJECT(button2), "clicked",
                      GTK_SIGNAL_FUNC (button_press_event),
                      (gpointer) 2); 

  button3 = gtk_button_new_with_label("C");
  gtk_box_pack_start (GTK_BOX (box2), button3, TRUE, TRUE, 0);
  gtk_widget_realize(button3);
  gtk_widget_show(button3); 
  gtk_signal_connect( GTK_OBJECT(button3), "clicked",
                      GTK_SIGNAL_FUNC (button_press_event),
                      (gpointer) 3); 

  button4 = gtk_button_new_with_label("D");
  gtk_box_pack_start (GTK_BOX (box2), button4, TRUE, TRUE, 0);
  gtk_widget_realize(button4);
  gtk_widget_show(button4);  
  gtk_signal_connect( GTK_OBJECT(button4), "clicked",
                      GTK_SIGNAL_FUNC (button_press_event),
                      (gpointer) 4); 

  gtk_widget_show(drawing_area);
  gtk_widget_show(box1);
  gtk_widget_hide(box2);
    
  gtk_widget_show(appwindow); 
  
  /* idle function */

  idle_deeds = gtk_idle_add((GtkFunction)grhythm_animate_strip, drawing_area); 

  gtk_main();
  
  return (0);
}
  
  






















